submodule: suppress checking for file name and ref ambiguity for object ids
The argv argument of collect_changed_submodules() contains only object ids (the objects references of all the refs). Notify setup_revisions() that the input is not filenames by passing assume_dashdash, so it can avoid redundant stat for each ref. Also suppress refname_ambiguity flag to avoid filesystem lookups for each object. Similar logic can be found in cat-file, pack-objects and more. This change reduces the time for git fetch in my repo from 25s to 6s. Signed-off-by: Orgad Shaneh <orgads@gmail.com> Signed-off-by: Junio C Hamano <gitster@pobox.com>
Orgad Shaneh committed
Sep 6, 2020 at 20:53 UTC
a462bee5d5b52c9778a9df600494fbd87f3c5fb9
1 file changed
+8
-1
submodule.c
+8
-1
@@ -840,9 +840,16 @@ static void collect_changed_submodules(struct repository *r,
840
{
841
struct rev_info rev;
842
const struct commit *commit;
843
+ int save_warning;
844
+ struct setup_revision_opt s_r_opt = {
845
+ .assume_dashdash = 1,
846
+ };
847
848
+ save_warning = warn_on_object_refname_ambiguity;
849
+ warn_on_object_refname_ambiguity = 0;
850
repo_init_revisions(r, &rev, NULL);
845
- setup_revisions(argv->nr, argv->v, &rev, NULL);
851
+ setup_revisions(argv->nr, argv->v, &rev, &s_r_opt);
852
+ warn_on_object_refname_ambiguity = save_warning;
853
if (prepare_revision_walk(&rev))
854
die(_("revision walk setup failed"));
855